@@ -2,14 +2,34 @@
 fancycompleter: colorful TAB completion for Python prompt
 """
 
-__version__='0.2'
+__version__='0.3'
 __author__ ='Antonio Cuni <anto.cuni@gmail.com>'
 __url__='http://bitbucket.org/antocuni/fancycompleter'
 
 import rlcompleter
 import types
 import os.path
-from itertools import izip, count
+from itertools import count
+from functools import reduce
+
+# python3 compatibility
+# ---------------------
+try:
+    from itertools import izip
+except ImportError:
+    izip = zip
+
+try:
+    from types import ClassType
+except ImportError:
+    ClassType = type
+
+try:
+    unicode
+except NameError:
+    unicode = str
+
+# ----------------------
 
 class Color:
     black = '30'
@@ -37,7 +57,6 @@ class Color:
             pass
         return '\x1b[%sm%s\x1b[00m' % (color, string)
 
-
 class DefaultConfig:
 
     consider_getitems = True
@@ -57,11 +76,11 @@ class DefaultConfig:
         types.FunctionType: Color.blue,
         types.BuiltinFunctionType: Color.blue,
         
-        types.ClassType: Color.fuchsia,
+        ClassType: Color.fuchsia,
         type: Color.fuchsia,
         
         types.ModuleType: Color.teal,
-        types.NoneType: Color.lightgray,
+        type(None): Color.lightgray,
         str: Color.green,
         unicode: Color.green,
         int: Color.yellow,
@@ -96,6 +115,11 @@ class DefaultConfig:
         if self.use_colors == 'auto':
             self.use_colors = supports_color
 
+def my_execfile(filename, mydict):
+    with open(filename) as f:
+        code = compile(f.read(), filename, 'exec')
+        exec(code, mydict)
+
 
 class ConfigurableClass:
     DefaultConfig = None
@@ -110,10 +134,10 @@ class ConfigurableClass:
         if os.path.exists(rcfile):
             mydict = {}
             try:
-                execfile(rcfile, mydict)
+                my_execfile(rcfile, mydict)
                 return mydict['Config']()
-            except Exception, e:
-                print '** error when importing %s: %s **' % (filename, e)
+            except Exception as e:
+                print('** error when importing %s: %s **' % (filename, e))
         return self.DefaultConfig()
 
 
@@ -134,6 +158,12 @@ class Completer(rlcompleter.Completer, ConfigurableClass):
         self.config = self.get_config(Config)
         self.config.setup()
         readline = self.config.readline
+        if hasattr(readline, '_setup'):
+            # this is needed to offer pyrepl a better chance to patch
+            # raw_input. Usually, it does at import time, but is we are under
+            # pytest with output captured, at import time we don't have a
+            # terminal and thus the raw_input hook is not installed
+            readline._setup()
         if self.config.use_colors:
             readline.parse_and_bind('set dont-escape-ctrl-chars on')
         if self.config.consider_getitems:
@@ -151,7 +181,7 @@ class Completer(rlcompleter.Completer, ConfigurableClass):
             return ['\t',None][state]
         else:
             return rlcompleter.Completer.complete(self,text,state)
-
+            
     def _callable_postfix(self, val, word):
         # disable automatic insertion of '(' for global callables: this method exists only in Python 2.6+
         return word
@@ -231,7 +261,11 @@ def commonprefix(names, base = ''):
 
 
 def has_leopard_libedit(config):
-    import commands
+    try:
+        import commands
+    except ImportError:
+        # python3? just return False for now
+        return False
     import sys
     # Detect if we are using Leopard's libedit.
     # Taken from IPython's rlineimpl.py.
@@ -264,7 +298,22 @@ def interact_pyrepl():
     sys.modules['readline'] = readline
     run_multiline_interactive_console()
 
-def interact():
+def setup_history(completer, persist_history):
+    import atexit
+    readline = completer.config.readline
+    #
+    if isinstance(persist_history, (str, unicode)):
+        filename = persist_history
+    else:
+        filename = '~/.history.py'
+    filename = os.path.expanduser(filename)
+    if os.path.isfile(filename):
+        readline.read_history_file(filename)
+    def save_history():
+        readline.write_history_file(filename)
+    atexit.register(save_history)
+
+def interact(persist_history=None):
     """
     Main entry point for fancycompleter: run an interactive Python session
     after installing fancycompleter.
@@ -286,6 +335,8 @@ def interact():
     """
     import sys
     completer = setup()
+    if persist_history:
+        setup_history(completer, persist_history)
     if completer.config.using_pyrepl and '__pypy__' not in sys.builtin_module_names:
         # if we are on PyPy, we don't need to run a "fake" interpeter, as the
         # standard one is fake enough :-)
